/*
 * Project Name:
     WiFi_AdHoc_Server
 * Copyright:
     (c) Mikroelektronika, 2012.

 * description  :
 *      this code shows how to use the Net_Wireless_MCW1001 library :
 *
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
     Dev.Board:       EasyMx v7 for STM32(R) ARM(R)
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HSE-PLL, 72.000MHz
     ext. modules:    ac:WIFI_Plus_Click on mikroBus2
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/eng/products/view/752/mikroc-pro-for-arm/
 * NOTES:
     - Pull-down PORTD.B0, PORTD.B1, PORTD.B2, PORTD.B3 at SW8, and put SW16.7 in VCC position (board specific)
     - Close pots RA3 at J8.
     - Turn ON PORTE/H LEDs at SW15.
     - Turn ON Buzzer at SW14.8
 */


sbit Net_Wireless_MCW1001_Chip_RST at GPIOC_ODR.B3; //easy mx pro v7 for stm32



////////////////////////////////////////////////////////////////////////////////
// Global variables

char myIpAdd[4]      = {169, 254, 0, 10};
char myMacAdd[6] = {0x22, 0x33, 0x44, 0x55, 0x66, 0x88};
char netMask[4]    = {255, 255, 0, 0};
char gatewayAdd[4] = {169, 254, 0, 1};
char remoteIpAdd[4];
unsigned int remotePort, localPort;
char strSSID[18] = "WiFi_AdHoc_Server";
char channels[11]  = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11};
char wifiStatus, bindResponse, backLog, listenResponse;
char socketHandle, socketChild;
char response;
unsigned int numOfSendBytes;
//
unsigned int DS1820_VALUE, LM35_VALUE;
char readbuff[10];
char writebuff[10];

////////////////////////////////////////////////////////////////////////////////
// Interrupt routine which increment Net_Wireless_MCW1001_Time variable every second.

unsigned int wifiTmr = 0;           // timer prescaler
void TIMER2_interrupt() iv IVT_INT_TIM2{
  TIM2_SR.UIF = 0;
  wifiTmr++;
  if(wifiTmr == 3) {                  // timer1 overflows 3 times per second
      Net_Wireless_MCW1001_Time++ ;  // increment WiFi library counter
      wifiTmr = 0;
  }
}

////////////////////////////////////////////////////////////////////////////////
// Initialization of MCU modules

void InitMcu() {

  // Initialization of timer..............
  RCC_APB1ENR.TIM2EN = 1;       // Enable clock gating for timer module 6
  TIM2_CR1.CEN = 0;             // Disable timer
  TIM2_PSC = 366;                  // Set timer prescaler. The counter clock frequency is equal to fCK_PSC / (PSC[15:0] + 1).
  TIM2_ARR = 65514;
  NVIC_IntEnable(IVT_INT_TIM2);     // Enable timer interrupt
  TIM2_DIER.UIE = 1;            // Update interrupt enable
  TIM2_CR1.CEN = 1;             // Enable timer

  // Set PE15 pin as digital output, so we can monitor TCP status on that pin
  GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_15);
  GPIOE_ODR.B15 = 0;
  ADC1_Init();
  Sound_Init(&GPIOE_ODR, 14);
  Net_Wireless_MCW1001_HwReset();

}

////////////////////////////////////////////////////////////////////////////////
// Initialization of WiFi module

void InitWiFi() {
  volatile char response;

  Net_Wireless_MCW1001_TimeToWait = 1;
  response = 1;
  while(response != 0)
    response = Net_Wireless_MCW1001_SetMode(_NET_WIRELESS_MCW1001_CP_1, _NET_WIRELESS_MCW1001_MODE_ADHOC); // Set "Connection Profile 1" mode (Ad-Hoc mode)
  
  Net_Wireless_MCW1001_TimeToWait = 10;
  response =Net_Wireless_MCW1001_SetChannelList(1, channels);                     // Choose only channerl one for communication
  response =Net_Wireless_MCW1001_SetSecurity_Open(_NET_WIRELESS_MCW1001_CP_1);    // Set open security level
  response =Net_Wireless_MCW1001_SetSSID(_NET_WIRELESS_MCW1001_CP_1, strSSID);    // Set SSID of network

  // Set network parameters
  Net_Wireless_MCW1001_SetIP(myIpAdd);
  response =Net_Wireless_MCW1001_SetNetworkMask(netMask);
  Net_Wireless_MCW1001_SetGatewayIP(gatewayAdd);
  Net_Wireless_MCW1001_SetMAC(myMacAdd);

  Net_Wireless_MCW1001_SetArpTime(1);                                   // Set gratuitous ARP timing
  Net_Wireless_MCW1001_SetRetryCount(5, 0);                             // No retries for the connection if the first attempt fails,
                                                                        // in order to establish a new network
}

unsigned int ReadDS1820(){
  unsigned int temp;
  // Perform temperature reading
  Ow_Reset(&GPIOA_BASE, 3);                         // Onewire reset signal
  Ow_Write(&GPIOA_BASE, 3, 0xCC);                   // Issue command SKIP_ROM
  Ow_Write(&GPIOA_BASE, 3, 0x44);                   // Issue command CONVERT_T
  Delay_ms(750);

  Ow_Reset(&GPIOA_BASE, 3);
  Ow_Write(&GPIOA_BASE, 3, 0xCC);                   // Issue command SKIP_ROM
  Ow_Write(&GPIOA_BASE, 3, 0xBE);                   // Issue command READ_SCRATCHPAD

  temp =  Ow_Read(&GPIOA_BASE, 3);
  temp = (Ow_Read(&GPIOA_BASE, 3) << 8) + temp;
  return temp;
}

////////////////////////////////////////////////////////////////////////////////
// Polling TCP receiving buffer to see is there some received packet

char CheckTcpReceive() {
  unsigned int numOfReceiveBytes = 0;
  char result=0;

  response = Net_Wireless_MCW1001_TCP_ReadBytes(&socketChild, 10, &readbuff, &numOfReceiveBytes);
  // If there is no error, and number of received bytes is greater than zero
  if((response == _NET_WIRELESS_MCW1001_NO_ERROR) && (numOfReceiveBytes == 10)) {
     GPIOE_ODR.B15 = ~GPIOE_ODR.B15 ;
     result=0xFF;
  }
  return result;
}

void Read(){
      writebuff[0]=0xF; //control byte
      writebuff[1]=0xF; //control byte
      //read temp. on DS1820
      DS1820_VALUE = ReadDS1820();
      //put value in output buffer
      writebuff[2]=(char)((DS1820_VALUE >> 8) & 0xFF);
      writebuff[3]=(char)(DS1820_VALUE & 0xFF);

      //read temp. on LM35
      LM35_VALUE = ADC1_Read(10);
      delay_ms(10);
      //put value in output buffer
      writebuff[4]=(char)((LM35_VALUE >> 8) & 0xFF);
      writebuff[5]=(char)(LM35_VALUE & 0xFF);
}

////////////////////////////////////////////////////////////////////////////////


void main() {

  InitMcu();
  ADC_Set_Input_Channel(_ADC_CHANNEL_10);

  UART2_Init_Advanced(115200, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_TWO_STOPBITS, &_GPIO_MODULE_USART2_PD56);
  Delay_ms(2000);

  InitWiFi();
  // Open "ad-hoc server"
  Net_Wireless_MCW1001_Connect(_NET_WIRELESS_MCW1001_CP_1, &wifiStatus);
  localPort = 10002;

  while(1) {
    socketHandle = 0;
    backLog = 1;
    Net_Wireless_MCW1001_SocketCreate(&socketHandle, _NET_WIRELESS_MCW1001_SOCKET_TYPE_TCP);   // Create TCP socket
    Net_Wireless_MCW1001_SocketBind(socketHandle, &localPort, &bindResponse);                  // Bind socket to the listen port
    Net_Wireless_MCW1001_TCP_Listen(socketHandle, &backLog, &listenResponse);                  // Prepare the socket to listen for connection
                                                                                               // with one children socket.
    while(1) {
      socketChild = socketHandle;
      Net_Wireless_MCW1001_TCP_Accept(&socketChild, &remotePort, remoteIpAdd);                 // Accept incoming conncetion
      if (socketChild != 254) {                                                                // Accept function set socketChild.
                                                                                               // If socketChild == 254 there is no incoming connection
        GPIOE_ODR.B15 = 1;                                                                     // Connection established
        Sound_Play(800, 200);            // Play sound at 800Hz for 40ms
        Delay_ms(100);
        Sound_Play(800, 200);            // Play sound at 800Hz for 40ms
        Delay_ms(1000);
        break;
      }
    }

    // Connection established
    while(1) {

      if(CheckTcpReceive()){
        if(readbuff[0] == 0xF){
          Read();
          response = Net_Wireless_MCW1001_TCP_SendBytes(socketChild, writebuff, 10, &numOfSendBytes);
        }
      }else{
        if(Net_Wireless_MCW1001_TCP_Status(socketChild, 0x1) != 0) {             // If connection closed improperly from remote side
           Net_Wireless_MCW1001_SocketClose(socketChild);                          // Close sockets
           Net_Wireless_MCW1001_SocketClose(socketHandle);
           GPIOE_ODR.B15 = 0;                                                      // Connection established
           Sound_Play(800, 200);                                                   // Play sound at 800Hz for 40ms
           Delay_ms(100);
           break;
        }
      
      
      }



    }

  }
}