﻿Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Timers
Imports GraphicsTable

Public Class Form1

    Private ServerAddress As String = "169.254.0.10"
    Private Port As Integer = 10002
    Private ClientSocket As TcpClient
    Private ServerStream As NetworkStream
    Private IsConnected As Boolean = False
    Private OutStream() As Byte
    Private RequestCommand() As Byte = {&HF, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0} '
    Private InputBuffer() As Byte
    Private OutputBufferSize As Integer = 10
    Private InputBufferSize As Integer = 10
    Private Shared tmrContinuousDataCollect As System.Timers.Timer
    Private TransferInProgress As Boolean = False

    Const lm35vref As Double = 1800 'mV
    Const lm35res As Double = 10 '10mV/C
    Const max12bit As Double = 4095


    Private Delegate Sub MarshalToForm(ByVal data() As Byte)

    Private Sub btConnect_Click(sender As Object, e As EventArgs) Handles btConnect.Click

        Try

            ClientSocket = New System.Net.Sockets.TcpClient(ServerAddress, Port)
            
            If (ClientSocket.Connected = True) Then

                ClientSocket.ReceiveBufferSize = InputBufferSize

                ClientSocket.SendBufferSize = OutputBufferSize

                ServerStream = ClientSocket.GetStream

                btConnect.Enabled = False

                btDisconnect.Enabled = True

                lblStatus.Text = "Connected"

                ExchangeInputAndOutputReports()

                tmrContinuousDataCollect.Start()

            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.OkOnly, "Connection error")
        End Try
       
    End Sub


    Private Sub ExchangeInputAndOutputReports()

        Dim success As Boolean = False

        Try
            If (ClientSocket.Connected = True) Then
                'send data to microcontroller
                If (ServerStream.CanWrite = True) Then
                    'Custom output data
                    'Dim outputBuffer() As Byte = New Byte(ClientSocket.SendBufferSize) {}
                    'ServerStream.Write(outputBuffer, 0, ClientSocket.SendBufferSize)
                    ServerStream.Write(RequestCommand, 0, ClientSocket.SendBufferSize)
                    success = True
                    ServerStream.Flush()
                End If
                'receive data from microcontroller
                If (success = True) Then
                    'To enable reading a report without blocking the main thread, this
                    'application uses an asynchronous delegate.
                    Dim ar As IAsyncResult = Nothing
                    TransferInProgress = True
                    InputBuffer = New Byte(ClientSocket.ReceiveBufferSize) {}
                    If (ServerStream.CanRead = True) Then
                        ServerStream.BeginRead(InputBuffer, 0, ClientSocket.ReceiveBufferSize, New AsyncCallback(AddressOf GetInputData), InputBuffer)
                    End If
                End If
            End If
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.OkOnly)
        End Try

    End Sub


    Private Sub GetInputData(ByVal ar As IAsyncResult)

        Dim inputReportBuffer As Byte() = Nothing

        Try
            inputReportBuffer = CType(ar.AsyncState, Byte())
            ServerStream.EndRead(ar)
            If (ar.IsCompleted = True) Then
                'Read data success
                MyMarshalToForm(inputReportBuffer)
            Else
                'todo
                'The attempt to read an Input report has failed
            End If

            TransferInProgress = False

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.OkOnly)
        End Try
    End Sub


    Private Sub MyMarshalToForm(ByVal data() As Byte)

        Dim args() As Object = {data}

        Dim MarshalToFormDelegate As MarshalToForm

        ' The AccessForm routine contains the code that accesses the form.

        MarshalToFormDelegate = New MarshalToForm(AddressOf AccessForm)

        ' Execute AccessForm, passing the parameters in args.

        MyBase.Invoke(MarshalToFormDelegate, args)

    End Sub

    Private Sub AccessForm(ByVal data() As Byte)


        If ((data(0) And data(1)) = &HF) Then 'check control bytes

            Dim tmp As UInt16 = 0

            tmp = data(2) And &HFF
            tmp = (tmp << 8) + data(3)
            Graph3.AddValue(getTempFromDS1820(tmp))

            tmp = 0
            tmp = data(4) And &HFF
            tmp = (tmp << 8) + data(5)
            Graph4.AddValue(getTempFromLM35(tmp))
        End If

    End Sub


    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles Me.Load
        btDisconnect.Enabled = False
        tmrContinuousDataCollect = New System.Timers.Timer(100)
        AddHandler tmrContinuousDataCollect.Elapsed, AddressOf OnDataCollect
        tmrContinuousDataCollect.Stop()
        tmrContinuousDataCollect.SynchronizingObject = Me
        Graph3.Title = "DS1820"
        Graph3.GraphicMode = Drawing2D.SmoothingMode.AntiAlias
        Graph4.Title = "LM35"
        Graph4.GraphicMode = Drawing2D.SmoothingMode.AntiAlias
    End Sub


    Protected Overrides Sub Finalize()

        If ClientSocket.Connected Then
            CloseConnection()
        End If

        MyBase.Finalize()

    End Sub

    Private Sub btDisconnect_Click(sender As Object, e As EventArgs) Handles btDisconnect.Click
        CloseConnection()
    End Sub

    Private Sub CloseConnection()
        Try

            tmrContinuousDataCollect.Stop()

            If (Not (ServerStream Is Nothing)) Then

                ServerStream.Close()

                ServerStream.Flush()

            End If

            ClientSocket.Close()

            btConnect.Enabled = True

            btDisconnect.Enabled = False

            lblStatus.Text = "Disconnected"

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
        
    End Sub


    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        CheckForIllegalCrossThreadCalls = False

    End Sub

    Private Sub OnDataCollect(ByVal source As Object, ByVal e As ElapsedEventArgs)
        Try

            If (TransferInProgress = False) Then
                ExchangeInputAndOutputReports()
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.OkOnly)
        End Try

    End Sub

    Private Function getTempFromLM35(ByVal Value As UInt16) As Double
        Return CDbl(Value) * lm35vref / max12bit / lm35res
    End Function

    Private Function getTempFromDS1820(ByVal Value As UInt16) As Double
        Dim a, b As Double
        a = CDbl(Value >> 4)
        b = CDbl(CDbl(Value And &HF) * 0.0625)
        Return a + b
    End Function

    
End Class
